// ImageWrapper.cpp: implementation of the ImageWrapper class.
//
//////////////////////////////////////////////////////////////////////

#include "ImageWrapper.h"
//#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include "main.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern char *GetPathFilename(char *filename);
extern unsigned char *loadJPG(char *path,int *width,int *height);

ImageWrapper::ImageWrapper()
{
	SetScale(1.0f);
}

ImageWrapper::~ImageWrapper()
{

}

void ImageWrapper::SetTexture(unsigned int id)
{
	texture_id=id;
	//	UpdateTexture();
}

// Set OpenGL texture from jpeg
static int UploadJPEG(char *path)
{
    int x,y,truecolor=0;
    unsigned char *pixels;

    pixels=loadJPG(path,&x,&y);
    if(pixels==NULL)
        return(-1);

    // Try to save texture memory by checking whether an image really is a
    // truecolor image.
    for(int n=0;n<x*y;n++)
    {
        if(pixels[n*3]!=pixels[n*3+1] || pixels[n*3+1]!=pixels[n*3+2])
        {
            truecolor=1;
            break;
        }
    }

    // The picture is loaded. Now dump it to GL.
    GLuint   lumero;
    glGenTextures(1,&lumero);
    glBindTexture(GL_TEXTURE_2D,lumero);
    glPixelStorei(GL_UNPACK_ALIGNMENT,1);

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);

    if(truecolor)
    {
        glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,x,y,0,GL_RGB,GL_UNSIGNED_BYTE,pixels);
    }
    else // For grayscale images we need only the luminance
    {
        unsigned char *img=new unsigned char[x*y];
        for(int n=0;n<x*y;n++)
            img[n]=pixels[n*3];

        glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE8,x,y,0,GL_LUMINANCE,GL_UNSIGNED_BYTE,img);
        delete(img);
    }

    delete(pixels);
    return(lumero);
}

void ImageWrapper::SetTexture(char *path)
{
	int i=UploadJPEG(path);
	texture_id=i;
	//if (i<0) { MessageBox(NULL, path, "Error loading", MB_OK); texture_id=0; }
	if (i<0) { fprintf(stderr,"Error loading %s\n",path); texture_id=0; }
}

void ImageWrapper::PaintTransparent()
{
		glDepthMask(GL_FALSE);
		glEnable(GL_BLEND);
			glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			Paint();
		glDisable(GL_BLEND);
		glDepthMask(GL_TRUE); 
}

void ImageWrapper::RawPaint()
{
	glBegin(GL_TRIANGLES);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-scale+px, -scale+py,  pz);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( scale+px, -scale+py,  pz);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( scale+px,  scale+py,  pz);	


		glTexCoord2f(1.0f, 0.0f); glVertex3f( scale+px,  scale+py,  pz);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-scale+px,  scale+py,  pz);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-scale+px, -scale+py,  pz);	
	glEnd();	

}

void ImageWrapper::Paint()
{
	glPushMatrix();
		glDepthMask(GL_FALSE);
//		glDisable(GL_CULL_FACE);

//		glColor4f( 1,1,1,0 );	

		glTranslatef(px,py,pz);						

//		glEnable(GL_BLEND);
//			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		
			glBindTexture(GL_TEXTURE_2D, texture_id);			
			glBegin(GL_QUADS);									
				glTexCoord2f(0.0f, 1.0f); glVertex3f(-scale, -scale,  0.0f);	
				glTexCoord2f(1.0f, 1.0f); glVertex3f( scale, -scale,  0.0f);	
				glTexCoord2f(1.0f, 0.0f); glVertex3f( scale,  scale,  0.0f);	
				glTexCoord2f(0.0f, 0.0f); glVertex3f(-scale,  scale,  0.0f);	
			glEnd();	
		
//		glDisable(GL_BLEND);

		glDepthMask(GL_TRUE); 
//		glEnable(GL_CULL_FACE);
	glPopMatrix();
}

void ImageWrapper::UpdateTexture()
{
}



void ImageWrapper::SetPosition(float x, float y, float z)
{
	px=x;
	py=y;
	pz=z;
}

ImageWrapper::ImageWrapper(char *path)
{
	SetTexture(path);
}

ImageWrapper::ImageWrapper(unsigned int id)
{
	SetTexture(id);
}

void ImageWrapper::SetScale(float s)
{
	scale = s;
}

void ImageWrapper::SetDefaults()
{
	SetScale(1.0f);
	SetPosition(0.0f, 0, -2.421f);
}
